/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm.attraction;

import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.topology.Topology;
import de.tud.kom.p2psim.impl.topology.util.PositionVector;
import de.tud.kom.p2psim.impl.util.oracle.GlobalOracle;
import de.tudarmstadt.maki.simonstrator.api.Binder;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.IAttractionPoint;

public class AbstractAttractionProvider implements IAttractionProvider {
	
	private LinkedList<AttractionPoint> attractionPoints = new LinkedList<>();
	protected PositionVector worldDimension;
	
	public AbstractAttractionProvider() {
		this.worldDimension = Binder.getComponentOrNull(Topology.class).getWorldDimensions();		
	}
	
	public LinkedList<AttractionPoint> getAttractionPoints() {
		return new LinkedList<AttractionPoint>(attractionPoints);
	}
	
	protected void clearAttractionPoints() {
		attractionPoints.clear();
	}
	
	protected boolean hasAttractionPoint(AttractionPoint ap) {
		return attractionPoints.contains(ap);
	}
	
	/**
	 * Remove an {@link IAttractionPoint} from the list
	 * @param ap
	 */
	public void removeAttractionPoint(AttractionPoint ap) {
		
		if(Monitor.hasAnalyzer(AttractionPointMonitor.class)) {
			Monitor.getOrNull(AttractionPointMonitor.class).removedAttractionPoint(ap);
		}
		
		GlobalOracle.removeAttractionPoint(ap);
		
		attractionPoints.remove(ap);
	}
	
	/**
	 * Add an {@link IAttractionPoint} to the list
	 * @param ap
	 */
	public void addAttractionPoint(AttractionPoint ap) {
		
		if(ap == null) {
			Monitor.log(IAttractionProvider.class, Level.ERROR, "IAttractionGenerator: Tried to add NULL as Attraction Point");
			return; 
		}			
		
		if(Monitor.hasAnalyzer(AttractionPointMonitor.class)) {
			Monitor.getOrNull(AttractionPointMonitor.class).addedAttractionPoint(ap);
		}
		
		GlobalOracle.addAttractionPoint(ap);
		
		attractionPoints.add(ap);
	}
}
