/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm.attraction;

import java.util.LinkedList;

import de.tud.kom.p2psim.impl.topology.util.PositionVector;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

public class MobileAttractionPoint extends AttractionPoint  {

	private LinkedList<PositionVector> waypoints;
	private double speed = 0.5;
	private boolean move = false;
	private long pausetime = Time.MINUTE * 10;
	private long timer = 0L;
	
	@XMLConfigurableConstructor({ "name", "x", "y" })
	public MobileAttractionPoint(String name, double x, double y) {
		super(name, x, y);
		
		waypoints = new LinkedList<PositionVector>();
		waypoints.add(new PositionVector(250,250));
		waypoints.add(new PositionVector(250,400));
		waypoints.add(new PositionVector(750,400));	
		waypoints.add(new PositionVector(750,250));	
	}
	
	public void move() {
		
		if(move) {
			PositionVector destination = waypoints.getFirst();
			
			if (destination.distanceTo(this) > speed) {
				this.set(this.moveStep(destination, speed));
			}
			else {
				this.set(destination);			
				waypoints.add(waypoints.removeFirst());
				
				timer = 0;
				move = false;
			}
		}
		else {
			timer += Time.SECOND;
			
			if(timer >= pausetime) {
				timer = 0;
				move = true;				
			}
		}
		
		
	}
}
