/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package de.tud.kom.p2psim.impl.topology.movement.modularosm.groups.groupencounter;

import de.tud.kom.p2psim.api.scenario.ConfigurationException;
import de.tud.kom.p2psim.impl.topology.movement.modularosm.groups.SocialMovementGroup;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;


/**
 * This class checks if there is any pair of groups which encounter each other. Randomly decides
 * if the groups merge partially or not after the encounter.
 * 
 * @author Marcel Verst
 * @version 1.0, 22.11.2018
 */
public class MergePartial extends AbstractGroupEncounter {
	//--- VARIABLES TO BE DECLARED IN CONFIG ---
	private double probabilityToMergePartially;
	private int numberOfHostsToLeave;
	//------------------------------------------

	@XMLConfigurableConstructor({"enableGroupEncounters","groupEncounterMeetingDistance","groupReencounterWaitTime"})
	public MergePartial(boolean enableGroupEncounters,
			double groupEncounterMeetingDistance,
			long groupReencounterWaitTime) {
		super(enableGroupEncounters, groupEncounterMeetingDistance,
				groupReencounterWaitTime);		
	}
	
	/**
	 * 
	 * Lets both groups merge partially with a certain probability in case they encounter.
	 */
	@Override
	public void handleGroupEncounter(SocialMovementGroup group1, SocialMovementGroup group2) {
		if(decide() == DECISION_MERGE_PARTIAL) {
			mergeGroupsPartially(group1, group2, numberOfHostsToLeave);
		}
	}
	
	public void setProbabilityToMergePartially(double probabilityToMergePartially) {
		if (probabilityToMergePartially < 0 || probabilityToMergePartially > 1.0) {
			throw new ConfigurationException(
					"probabilityToMergePartially should be between 0.0 and 1.0!");
		}
		this.probabilityToMergePartially = probabilityToMergePartially;
	}

	public void setNumberOfHostsToLeave(int numberOfHostsToLeave) {
		if (numberOfHostsToLeave < 0) {
			throw new ConfigurationException(
					"numberOfHostsToLeave should be greater or equal than 0!");
		}
		this.numberOfHostsToLeave = numberOfHostsToLeave;
	}
	
	protected int decide() {
		// mergePartially
		if(rand.nextDouble() < probabilityToMergePartially) {
			return DECISION_MERGE_PARTIAL;
		}		
		else 
			return DECISION_NONE;
	}
}
