/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm.transition;

import de.tud.kom.p2psim.api.topology.movement.SimLocationActuator;
import de.tudarmstadt.maki.simonstrator.api.Event;
import de.tudarmstadt.maki.simonstrator.api.EventHandler;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.IAttractionPoint;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

/**
 * A simplified transition strategy taking only the weights of an AP into account.
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, Jan 16, 2017
 * 
 * Made the transition strategy extend the {@link AbstractAttractionBasedAssignmentStrategy} for a more generalized approach on 
 * the {@link IAttractionAssigmentStrategy} implementation for attraction point based strategies.
 * 
 * @author Julian Zobel
 * @version 1.1, January 2019
 * 
 */
public class WeightedTransitionStrategy extends AbstractAttractionBasedAssignmentStrategy implements EventHandler {
			
	@XMLConfigurableConstructor({ "defaultPauseTimeMin", "defaultPauseTimeMax" })
	public WeightedTransitionStrategy(long defaultPauseTimeMin, long defaultPauseTimeMax) {
		super(defaultPauseTimeMin, defaultPauseTimeMax);
	}
	
	@Override
	public void addComponent(SimLocationActuator ms) {
		IAttractionPoint nextAP = getNewAttractionPointAssignment(ms);
		updateTargetAttractionPoint(ms, nextAP);		
	}
			
	@Override
	public void eventOccurred(Object content, int type) {
		assert type == EVENT_PAUSE_ENDED;
		SimLocationActuator comp = (SimLocationActuator) content;
		if (!this.assignments.containsKey(comp)) {
			// maybe someone assigned a new AP in the meantime. ignore.
			this.assignments.put(comp, getNewAttractionPointAssignment(comp));
		}
	}

	@Override
	public void reachedAttractionPoint(SimLocationActuator ms, IAttractionPoint attractionPoint) {	
		// JZ: Added this to prevent multiple calls while pausing (otherwise most nodes will not wait more the minimum pause time)
		if (!this.assignments.containsKey(ms)) {			
			return;
		}		
		
		this.lastAssignments.put(ms, this.assignments.remove(ms));
		Event.scheduleWithDelay(getPauseTime(attractionPoint), this, ms, EVENT_PAUSE_ENDED);
	}

}
